@@ -52,7 +52,6 @@
 static gboolean parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *root_node, gpointer user_data, GError **error);
 
 G_DEFINE_TYPE (GDataDocumentsFeed, gdata_documents_feed, GDATA_TYPE_FEED)
-#define GDATA_DOCUMENTS_FEED_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GDATA_TYPE_DOCUMENTS_FEED, GDataDocumentsFeedPrivate))
 
 static void
 gdata_documents_feed_class_init (GDataDocumentsFeedClass *klass)
@@ -99,15 +98,15 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 		GDataEntry *entry = NULL;
 		gchar *kind = get_kind (doc, node);
 
-		if (g_strcmp0 (kind, "http://schemas.google.com/docs/2007#spreadsheet") == 0)
+		if (g_strcmp0 (kind, "http://schemas.google.com/docs/2007#spreadsheet") == 0) {
 			entry = GDATA_ENTRY (_gdata_parsable_new_from_xml_node (GDATA_TYPE_DOCUMENTS_SPREADSHEET, doc, node, NULL, error));
-		else if (g_strcmp0 (kind, "http://schemas.google.com/docs/2007#document") == 0)
+		} else if (g_strcmp0 (kind, "http://schemas.google.com/docs/2007#document") == 0) {
 			entry = GDATA_ENTRY (_gdata_parsable_new_from_xml_node (GDATA_TYPE_DOCUMENTS_TEXT, doc, node, NULL, error));
-		else if (g_strcmp0 (kind, "http://schemas.google.com/docs/2007#presentation") == 0)
+		} else if (g_strcmp0 (kind, "http://schemas.google.com/docs/2007#presentation") == 0) {
 			entry = GDATA_ENTRY (_gdata_parsable_new_from_xml_node (GDATA_TYPE_DOCUMENTS_PRESENTATION, doc, node, NULL, error));
-		else if (g_strcmp0 (kind, "http://schemas.google.com/docs/2007#folder") == 0)
+		} else if (g_strcmp0 (kind, "http://schemas.google.com/docs/2007#folder") == 0) {
 			entry = GDATA_ENTRY (_gdata_parsable_new_from_xml_node (GDATA_TYPE_DOCUMENTS_FOLDER, doc, node, NULL, error));
-		else {
+		} else {
 			g_message ("%s documents are not handled yet", kind);
 			g_free (kind);
 			return TRUE;
@@ -120,6 +119,7 @@ parse_xml (GDataParsable *parsable, xmlDoc *doc, xmlNode *node, gpointer user_da
 		/* Call the progress callback in the main thread */
 		_gdata_feed_call_progress_callback (GDATA_FEED (self), user_data, entry);
 		_gdata_feed_add_entry (GDATA_FEED (self), entry);
+		g_object_unref (entry);
 
 		return TRUE;
 	}
